/* This file contains all functions used by the kiwifruit shoot L-system model. */
/* If you make any changes, don't forget to save it before running the model. */

#ifndef _FUNCTIONS_H
#define _FUNCTIONS_H

#include <math.h>

/* -------------------------------------------------------------------------- */
// common sink/source functions

float LogisticGrowthRate (float dev_age)
// defines a logistic growth rate based on developmental age
{
  return (1.0 / (1.0 + exp(dev_age))); 
}

float SinkUnloading (float c, float vm, float km)
// defines carbon unloading
{
  return (c / (km + c));
}

float dSinkUnloading_dc (float c, float vm, float km)
// defines the derivative with respect to 'c' of carbon unloading function
{
  return (km / ((km + c)*(km + c)));
}

float SourceLoading (float c, float km)
// c is carbon concentration (g C / cm^3)
{
  return (-1.0 + c / (km + c));
}

float dSourceLoading_dc (float c, float km)
{
  return (km / ((km + c) * (km + c)));
}


/* -------------------------------------------------------------------------- */
// apex specific functions

float ApexRateofAppearance (void)
{
  // phyllochron should be a function of temperature
  return (1.0 / PHYLLOCHRON);
}

int ApexGrowthCessation (float c, float vigour, float node)
// where 'c' is the hydrostatic pressure outside the apex
// and 'vigour' is the shoot's growth potential
{
  const float NUMBER_PREFORMED_NODES = 18.f;

  // if concentration is less than 0.1, check for growth cessation
#if SHOOT_TIP_ABORTION
  if (c <= CARBON_THRESHOLD)
  {
    Printf ("AN APEX ABORTED\n");
    return (1);
  }
#endif

  // growth cessation based on vigour
  if (node <= NUMBER_PREFORMED_NODES)
  {
    if (ran(1) > (1.f-PROB_PREFORMED_CESSATION)*vigour)
      return (1);
  }
  else
  {
    if (ran(1) > (1.f-PROB_NEOFORMED_CESSATION)*vigour)
      return (1);
  }

  return (0);
}


/* -------------------------------------------------------------------------- */
// leaf specific functions

float LeafSpecificGrowthRate (void)
{
  return (1.0 / TAU_LEAF);
}

float LeafLowerAsymptote (float node, float order)
// defines the lower asymptote of the Boltzmann function (like logistic but with extra parameter)
{
  static float B[17] =
    {-6.515,-12.772,-18.733,-24.353,-29.572,-34.322,
     -38.532,-27.439,-19.677,-13.848,-9.586,-6.539,
     -4.402,-2.928,-1.927,-1.264,-0.865};

  const float scale_factor = 0.45;

  if (node <= 17 && order <= 1)
    return (B[(int)node-1]*LEAF_BIO2CARB*LEAF_DENSITY*scale_factor);
  else
    return (B[16]*LEAF_BIO2CARB*LEAF_DENSITY*scale_factor);
}

float LeafPotentialGrowthRate (float carbon, float dev_age, float node, float order)
// parameter 'carbon' is leaf structural carbon
{
  return ((carbon - LeafLowerAsymptote(node,order)) * LogisticGrowthRate(dev_age) / TAU_LEAF);
}

float LeafMaintenance (float carbon)
// parameter 'carbon' is leaf structural carbon
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  // QQQ TYPO: INDE! return(carbon * MTNCE_INDE * exp(temp_response * (temp_avg-20.0)));
  return(carbon * MTNCE_LEAF * exp(temp_response * (temp_avg-20.0)));
}

float LeafCarbonOutput (float carbon)
// 'carbon' is carbon accumulated from photosynthesis
{
  return (LEAF_LOADING * carbon);
}

float LeafCarbonFromPhoto (float carbon, float structural_carbon, float acquired_carbon, float area)
// returns the amount of carbon acquired in leaf from photosynthesis
// carbon gives what is already there (stored as starch?)
// structural_carbon is size of the leaf (sink) in g C
// acquired_carbon is g C / (cm^2 day), so area is also the size of the leaf
{
  const float MAX_STORE = 1.0;
  const float SYNTHESIS_RATE = 1.0;

  if (acquired_carbon * area * DT + carbon > MAX_STORE*structural_carbon)
    return (MAX_STORE*structural_carbon - carbon);
  else
    return SYNTHESIS_RATE*(acquired_carbon * area);
}

float Carb2Area (float q)
{
  // return area of leaf given amount of carbon
  return (q / (LEAF_BIO2CARB*LEAF_DENSITY));
}

float LeafInitialDevAge (float node, float order)
// defines the initial developmental age of a leaf by node number
{
  static float alpha[17] =
    {-1.964,-1.983,-2.003,-2.023,-2.043,-2.063,-2.083,
     -2.531,-2.951,-3.371,-3.790,-4.210,-4.630,-5.049,
     -5.469,-5.889,-6.233};

  if (order == 1)
  {
    if (node <= 17)
      return (alpha[(int)node-1]);
    else 
      return (alpha[16]+(node-17)*0.00461);
  }
  else // all leaves are preformed
    return (alpha[16]+node*0.00461);
}


/* -------------------------------------------------------------------------- */
// internode specific functions

float IndeSpecificGrowthRate (void)
{
  return (1.0 / TAU_INDE);
}

float IndePrimaryGrowthRate (float dev_age, float length)
{
  return (LogisticGrowthRate(dev_age) * length / TAU_INDE);
}

float IndeSecondaryGrowthRate (float dev_age, float radius)
{
  return ((SEC_GROWTH_RATE +
           SEC_LOGISTIC_RATE * LogisticGrowthRate(dev_age) * radius) / TAU_INDE);
}

float IndeMaintenance (float carbon, float order)
// parameter 'carbon' is structural carbon
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  if (order == TRUNK_ORDER)
    return(carbon * MTNCE_TRUNK * exp(temp_response * (temp_avg-20.0)));
  else if (order == LEADER_ORDER)
    return(carbon * MTNCE_LEADER * exp(temp_response * (temp_avg-20.0)));
  else if (order == CANE_ORDER)
    return(carbon * MTNCE_CANE * exp(temp_response * (temp_avg-20.0)));
  else
    return(carbon * MTNCE_INDE * exp(temp_response * (temp_avg-20.0)));
}

float IndeReserveSynthesis (float carbon, float stored_carbon, float syn_rate)
// where 'carbon' is structural carbon
// and 'stored_carbon' is the carbon in reserves
{
  if (carbon*INDE_RESERVES < stored_carbon)
    Printf ("Warning: internode stored carbon is > maximum possible!\n");

  return (syn_rate * (carbon*INDE_RESERVES - stored_carbon));
}

float IndeReserveHydrolysis (float stored_carbon, float hyd_rate)
{
  return (stored_carbon*hyd_rate);
}

float IndeFlowResistance (float length, float radius, float resistance)
{
  return (resistance * length / (2.0 * M_PI * radius));
}

float IndeInitialLength (float node, float order)
{
  const float scale_factor = 0.45;

  if (node <= 21)
    return (0.24+0.105*(node-1)) * scale_factor;
  else
    return (115-0.72*node)/45.0 * scale_factor;
}

float IndeInitialDevAge (void)
// defines the initial developmental age of an internode
{
  return (-INDE_TIMING_EXPANSION/TAU_INDE);
}


/* -------------------------------------------------------------------------- */
// flower/fruit specific functions

int FlowerDevelopment (float node)
// returns 1 (true) if flower should develop at this node number, else 0 (false)
{
  // probabilities taken from Walton and Fowke(1993):
  // Fig. 1 (top), shaded bars, nodes 5 to 12, as we assume only 8 floral nodes
  float prob_floral_develop[8] = {0.05, 0.2, 0.4, 0.7, 0.8, 0.65, 0.5, 0.22};
  int nodei = (int) (node - 1.0);

  if (node <= 8.0)
  {
    if (ran(1) < prob_floral_develop[nodei])
      return (1);
  }

  return (0);
}

float FlowerGrowthRate (float final_carbon)
{
  return (final_carbon / MAX_FLOWER_AGE);
}

float FruitGrowthRate (float carbon)
// growth rate function from Gandar, Hall and de Silva (19??).
{
  float k1 = 0.020749;
  float k2 = 22.213;
  float k3 = 0.63708;

  float norm = carbon / MAX_FRUIT_CARBON;

  return MAX_FRUIT_CARBON * (k1 * norm * (1.0 - norm) * (1.0 + k2*(norm-k3)*(norm-k3)));
}

float FruitMaintenance (float carbon)
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  return(carbon * MTNCE_FRUIT * exp(temp_response * (temp_avg-20.0)));
}


/* -------------------------------------------------------------------------- */
// cane specific functions

float CaneIndeInitialLength (float node)
{
  return (IndeInitialLength(node,0)*CANE_INTERNODE_LEN);
}

float CaneIndeInitialRadius (float node, float num_nodes)
{
  return ((0.3-CANE_INTERNODE_RAD)/num_nodes*(node) + CANE_INTERNODE_RAD);
}

float CaneIndeInitialReserves (float structural_carbon)
{
  // INDE_RESERVES=0.0592 is from 20/9 gDW / gC * 4/9 gC / gStarch * 3/50 gStarch/ gDW
  // the last constant is from Greaves et al (1999)

  return (structural_carbon * INDE_RESERVES);
}


/* -------------------------------------------------------------------------- */
// roots specific functions

float RootsGrowthRate (float structural_carbon)
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  return (structural_carbon * ROOTS_GROWTH_RATE * exp(temp_response * (temp_avg-20.0)));
}

float RootsMaintenance (float carbon)
{
  const float temp_response = log(2.0) / 10.0;
  const float temp_avg = 0.5 * (12.0 + 17.0);

  return(carbon * MTNCE_ROOTS * exp(temp_response * (temp_avg-20.0)));
}

float RootsReserveSynthesis (float carbon, float stored_carbon, float syn_rate)
// where 'carbon' is structural carbon
// and 'stored_carbon' is the carbon in reserves
{
  if (carbon*ROOTS_RESERVES < stored_carbon)
    Printf ("Warning: roots stored carbon is > maximum possible!\n");

  return (syn_rate * (carbon*ROOTS_RESERVES - stored_carbon));
}

float RootsReserveHydrolysis (float stored_carbon, float hyd_rate)
{
  return (stored_carbon*hyd_rate);
}

float RootsInitialReserves (float structural_carbon)
{
  // ROOTS_RESERVES=0.217 is from 1/0.45 gDW / gC * 0.444 gC / gStarch * 0.225 gStarch/ gDW
  // the last constant is from Smith et al (1992) for fibrous roots

  return (structural_carbon * ROOTS_RESERVES);
}


/* -------------------------------------------------------------------------- */
// photosynthesis functions

float Laing1985 (float flux_density)
{
  // photosynthesis from Laing (1985)
  const float Pmax = 836.0;  // micro grams CO2 / m^2 s
  const float Rd = 35.0;     // micro grams CO2 / m^2 s
  const float Ihalf = 334.0; // micro mol PAR / m^2 s
  const float Imax = 650.0;  // micro mol PAR / m^2 s

  float photosynthesis = Pmax * flux_density*Imax / (Ihalf + flux_density*Imax - Rd);

  // in untis of (g C cm^-2 day^-1)
  float acquired = photosynthesis * 0.0864 * 0.2727 * 0.0001;

  return (acquired);
}

float Greer2004 (float flux_density)
{
  // photosynthesis from Greer et al (2004)
  const float Pmax = 15.2;   // micro mol CO2 / m^2 s
  //const float Rd = 0.714;  // only needed for net photosynthesis (if you don't have respiration for a leaf already)
  const float app = 0.039;   // mol CO2 / mol PAR
  const float Imax = 1450.0; // micro mol PAR / m^2 s

  float photosynthesis = Pmax * (tanhf(flux_density*Imax*app/Pmax));

  return (photosynthesis * 0.000103772448); // gC / (cm^2 day)
}


/* -------------------------------------------------------------------------- */
// shoot vigour functions

// vigour function parameters
const float MAX_VIGOUR = 0.99f; // upper asymptote
const float MIN_VIGOUR = 0.f; // lower asymptote
const float INFLECTION_NODE1 = 1.f;
const float INFLECTION_NODE2 = 30.f;
const float TAU1 = 1.0f;
const float TAU2 = 2.0f;

// randomization of vigour using normal distribution
const float MEAN_VIGOUR = 0.95;
const float STD_VIGOUR = 0.24;

const float INITIAL_CLUSTER_MEAN = 5.7; // mean number of nodes in initial cluster
const float INITIAL_CLUSTER_STD = 2.1;  // std of nodes in initial cluster
const int MAX_SHORT_NODES = 9;

float BinomialRand (int n, float p)
/* an inefficent but direct implementation for generating binomial deviates */
{
  float bnl;

  bnl=0.f;  
  for (int i = 0; i < n; i++)
    if (ran(1) < p)
      bnl += 1.f; 
  return (bnl);
}

float NormalRand (void)
/* generates a random variate with normal distribution */
{
  static int done = 0;
  float x1, x2, w;
  static float y1, y2;

  if (done == 0)
  {
    do {
      x1 = 2.0 * ran(1) - 1.0;
      x2 = 2.0 * ran(1) - 1.0;
      w = x1 * x1 + x2 * x2;
    } while (w >= 1.f || w==0.f);

    w = sqrt(-2.0 * log(w) / w);
    y1 = x1 * w;
    y2 = x2 * w;

    done = 1;
    return y1;
  }
  else
  {
    done = 0;
    return y2;
  }
}

float ShootVigour (float node)
// defines the bud/shoot vigour at each node along the cane
// it is the product of two boltzmann functions
{
  float upper_asymptote;
  float lower_asymptote;
  float n0;
  float tau;
  float boltzmann[2];

  upper_asymptote=MAX_VIGOUR;
  lower_asymptote=MIN_VIGOUR;

  n0=INFLECTION_NODE1;
  tau=TAU1;
  boltzmann[0] = (upper_asymptote-lower_asymptote)*exp((node-n0)/tau) /
    (1 + exp((node-n0)/tau)) + lower_asymptote;

  n0=INFLECTION_NODE2;
  tau=TAU2;
  boltzmann[1] = (upper_asymptote-lower_asymptote)*exp((node-n0)/tau) /
    (1 + exp((node-n0)/tau)) + lower_asymptote;

  return(boltzmann[0]*(1.f-boltzmann[1]));
}

float SetShootVigour (float node_number)
{
  float vigour = (NormalRand()*STD_VIGOUR + MEAN_VIGOUR) * ShootVigour(node_number-1);
  if (vigour > MAX_VIGOUR)
    vigour = MAX_VIGOUR;
  else if (vigour < MIN_VIGOUR)
    vigour = MIN_VIGOUR;
  
  //float prob_short_shoot = 1.f - PROB_SHOOT_DEVELOPMENT*vigour;
  float prob_short_shoot = 1.f - PROB_SHOOT_DEVELOPMENT;
  if (ran(1) <= prob_short_shoot)
    vigour = 0.0;

  return (vigour);
}

int SetInitialClusterSize (void)
{
  // create a shoot with some number of metamers already formed (initial cluster)
  float p = 1.f - INITIAL_CLUSTER_STD/INITIAL_CLUSTER_MEAN;
  int n = (int) floor(INITIAL_CLUSTER_MEAN / p);
  int num_nodes = (int) BinomialRand(n,p);

  if (num_nodes > MAX_SHORT_NODES)
    num_nodes = MAX_SHORT_NODES;
  return (num_nodes);
}

/* -------------------------------------------------------------------------- */

#endif
