#include <lpfgall.h>

#include "modeloutput.h"

// defines pertinent to computation
const float INFTY = 1e11;        // very large number
const float EPSILON = 1.0/INFTY; // very small number different from zero
const float DT = 0.1;            // time step (days)
const float MAXERROR = 1e-4;     // error threshold for evaluating carbon flow
const float ENV_STEP = 1.0;      // when to compute light distribution (days) (keep as 1 for now!!!)
const float VIS_STEP = 1.0;      // when to visualise model (days)
const float OUT_STEP = 1.0;    // when to write model output to a file

#define MTNCE_ABSCISSION   0 // if set to 0, abscission due to insufficient supply for maintenance respiration is off, else it is on 
#define SHOOT_TIP_ABORTION 1 // if set to 1, shoot tip aborts if local carbon supply falls below threshold
#define TBAR               1 // if set to 1, the canes are trained on a T-bar support structure, else a pergola
#define SINGLE_SHOOT       0 // if set to 1, the model is of a single shoot, otherwise the whole vine
#define USE_QUASIMC        0 // if set to 1, the light environment program, quasiMC, will be used to compute absorbed irradiance per leaf

// defines pertinent to architectural model
#if SINGLE_SHOOT
const float PROB_PREFORMED_CESSATION = 0.0;
const float PROB_NEOFORMED_CESSATION = 0.0;
const float PROB_SHOOT_DEVELOPMENT = 0.0;
#else
// A. Deliciosa parameters
const float PROB_PREFORMED_CESSATION = 0.005; 
const float PROB_NEOFORMED_CESSATION = 0.09;
const float PROB_SHOOT_DEVELOPMENT = 0.98; 

// A. Chinensis parameters
//const float PROB_PREFORMED_CESSATION = 0.1; 
//const float PROB_NEOFORMED_CESSATION = 0.01; 
//const float PROB_SHOOT_DEVELOPMENT = 0.65; 
#endif

// defines pertinent to the permanent structure
const int TRUNK_NUM_NODES = 10;
const float TRUNK_INDE_LEN = 18.0; // cm: 10 nodes * 18 = 180 cm trunk
const int NUM_CANES = 28;
const float MTNCE_TRUNK = 0.000184; // trunk maintenance coefficient (g C / g C day)
const float MTNCE_LEADER = 0.000184;
const float TRUNK_ORDER = 0;
const float LEADER_ORDER = TRUNK_ORDER+1;

// defines pertinent to kiwifruit cane
const float MIN_BUD_BREAK_ANGLE=0.1; // value in [0,1) with 0 as 90 degrees from initial up vector
const int CANE_NUM_NODES = 30;
const float CANE_INTERNODE_LEN = 10.0; // 'relative' max length of cane internode (cm)
const float CANE_INTERNODE_RAD = 1.0; // max initial radius of cane internode (cm)
const float MTNCE_CANE = 0.00184; // cane maintenance coefficient (g C / g C day)
const float CANE_ORDER = LEADER_ORDER+1;

// defines pertinent to kiwifruit shoot
const float MAX_NODES = 100.0;    // maximum number of nodes on a shoot
const float MAX_GROWTH = 240.0;  // max growing days after budbreak (days)
const float AX_SHOOT_ORDER = CANE_ORDER+1;
const float REGROWTH_DELAY = 0.0; // the delay in start of the regrowth after pruning (days)
const float REGROWTH_VIGOUR = 1.0;

// defines pertinent to leaves
const float TAU_LEAF = 5.53;            // size of leaf 'growth window' (days)
const float LEAF_DENSITY = 0.00819;     // biomass to area (g DW / cm^2)
const float LEAF_BIO2CARB = 0.45;       // biomass to carbon (g C / g DW)
const float MTNCE_LEAF = 0.00806;         // leaf maintenace respiration co-efficient (g C / g C day)
// 378 mg carbohydrate kg^-1 DM h^-1 = (0.378g*0.4) gC * 1g DM / 450g C * 24h/d = 0.00806 gC/gC/day
// (assuming carbohydrate is glucose: C6H12O6: 6*12+1*12+16*6=180 molecular weight ==> 6*12 C / 180 = 0.4)
const float INCLINATION_ANGLE = 33.0;   // a leaf's inclination angle
const float PHYLLOTAXIS = 144.0;        // the phyllotactic angle
const float LEAF_LOADING = 1.0;         // rate of carbon loading (dimensionless)
const float LEAF_INITIAL_CARBON = 0.1*LEAF_DENSITY*LEAF_BIO2CARB; // initial carbon in a leaf (g C) at time of appearance

// defines pertinent to internodes
const float TAU_INDE = TAU_LEAF*0.5;      // size of internode 'growth window' (days)
const float INTERNODE_DENSITY = 0.189429; // carbon to volume (g C / cm^3)
const float MTNCE_INDE = 0.00527;          // internode maintenace respiration co-efficient (g C / g C day)
const float INDE_TIMING_EXPANSION = 6.0;  // inflection point for elongation of internodes (day)
const float SEC_GROWTH_RATE = 0.003;//0.0015;    // coefficient of proportionality for secondary growth (dimensionless)
const float SEC_LOGISTIC_RATE = 0.28;     // logisitic growth rate for young internodes (dimensionless)
const float INDE_RESERVES = 0.0592596;    // ratio of carbon reserves to structural carbon for mature internodes
const float INITIAL_INDE_RADIUS = 0.15;   // initial internode radius (cm)

// defines pertinent to apex
const float PHYLLOCHRON = 2.9;      // age at which the apex produces a metamer
const float CARBON_THRESHOLD = 0.01; // carbon concentration threshold for production of new metamers (g C / cm^3 ?)

// defines pertinent to roots
const float ROOTS_GROWTH_RATE = 0.003; // g C / (g C day)
const float MTNCE_ROOTS = 0.00184; // roots maintenance respiration coefficient (g C / g C day)
const float ROOTS_DENSITY = 5.0; // carbon (g C) to volume (cm^3) (this is just for visualisation)
//const float ROOTS_RESERVES = 0.1185192; // ratio of carbon reserves to structural roots
const float ROOTS_RESERVES = 0.2172852; // ratio of carbon reserves to fibrous roots (98% of root length in kiwifruit)
const float INIT_ROOTS_STORAGE = 6588.0; // (g C): Smith et al : 10.62 kg DW (fibrous) + 4.02 kg DW (structural) = 14.64 kg DW * 450 g C / kg DW = 6588 g C

// defines pertinent to flowers and fruit
const float MAX_FLOWER_AGE = 60.0; // days for flower to fully open (or accumulate INITIAL_FRUIT_CARBON)
const float FLOWER_FINAL_DIAMETER = 6.8; // maximum final diameter of a flower (cm)
const float FLOWER_DENSITY = 6.8 / 0.017; // cm per g C in flower (this is just for visualisation)
const float INITIAL_FLOWER_CARBON = 0.0; // initial carbon in a flower

const float MAX_FRUIT_AGE = 180.0; // days before fruit harvest (after anthesis)
const float FRUIT_DENSITY = 6.95; // cm^3 per g C in fruit (this is just for visualisation)
const float MTNCE_FRUIT = 0.00143; // fruit maintenace respiration co-efficient (g C / g C day)
const float MAX_FRUIT_DW = 32.3;//32.3; // maximum fruit dry weight at saturation (g DW) (See Minchin et al (2009) and Snelgar et al (1986))
const float MAX_FRUIT_CARBON = MAX_FRUIT_DW * 0.45; // maximum DW * g C / g DW
const float INITIAL_FRUIT_CARBON = 0.032; // initial carbon in fruit


// Definition of functions and structures for C-TRAM model

#include "functions.h"

// enumeration of sink/source module types
enum
{
  apex,
  internode_primary,
  internode_maintenance,
  internode_reserves,
  internode_hydrolysis,
  leaf_primary,
  leaf_maintenance,
  leaf_source,
  fruit_primary,
  fruit_maintenance,
  flower_primary,
  roots_primary,
  roots_maintenance,
  roots_reserves,
  roots_hydrolysis,
  begin,
  end,
  inert
};

// Declaration of the structure instantaneously representing a linearized organ
struct LinearData
{
   float e; // EMF
   float r; // resistance
};

// Function for connecting circuits in parallel
LinearData in_parallel(LinearData c1, LinearData c2)
{
  LinearData c;
  c.e = (c1.e * c2.r + c2.e * c1.r) / (c1.r + c2.r);
  c.r = (c1.r * c2.r) / (c1.r + c2.r);
  return c;
}

// Declaration of the structure that characterizes a sink/source
struct SegmentData
{
  LinearData cp;  // parallel part of the circuit: EMF and res. to ground
  float rs;	  // resistance in series
  LinearData ceq; // Equivalent circuit: resultant EMF and internal resistance
  float i;	  // current through the resistance rs
  float v;	  // potential at the (right) node
  float err;	  // current difference between iteratations
  float q;	  // accumulated charge in segment
  int type;	  // sink-source type: storage, primary, begin, end, possibly other

  void update_charge();
};

// function to update the carbon in sink/source depending on carbon flow
float total_flow_in;
float total_flow_out;
void SegmentData::update_charge()
{
  // update total flow (to check for errors)
  if (i >= 0)
    total_flow_in += i*DT;
  else
    total_flow_out += i*DT;

  q += i * DT; // update accumulated charge
  if (q < 0.0)
    q = 0.0;

  return;
}


// Enumeration of computation phases
#define Develop         1
#define UpdateDemand    2
#define PropagateLeft   3
#define PropagateRight 	4
#define UpdateCarbon    5
#define Debug           6
#define Output          7
#define LightInterface  8
#define Photosynthesis  9
#define Management      10


// Declarations of global variables 
int Phase;          // the current computation phase
float error;        // the error in computation, for measuring convergence to the solution
int error_count;    // the number of times 'error' is below MAXERROR
float sim_time;     // the current simulation time (steps of DT)
float env_time;     // the time to calculate light distribution (once a day)
float vis_time;     // the time to visualise the model
float out_time;     // the time to write model output to a file;

ModelOutput fruit_carbon_output("fruit_carbon.dat", NUM_CANES+1, 100, COL_MAJOR); // 100 fruit max per cane
ModelOutput node_number_output("node_number.dat", 500, 1, ROW_MAJOR); // 500 shoots max

// parameters to be fitted
// sink priorities, the priority coefficient
float KM_LEAF_PRI = 0.01;      // for primary leaf growth
float KM_LEAF_MAN = 0.00001;   // for leaf maintenance respiration
float KM_INDE_PRI = 0.01;      // for primary internode growth
float KM_INDE_SEC = 0.05;      // for secondary internode growth
float KM_INDE_MAN = 0.00001;   // for internode maintenance respiration
float KM_RESERVES = 0.1;       // for starch synthesis in internodes
float KM_FLOWER = 0.01;        // for flower growth
float KM_FRUIT_PRI = 10.0;     // for fruit growth
float KM_FRUIT_MAN = 0.00001;  // for fruit maintenance
float KM_ROOTS = 30.0;         // for root growth
float KM_ROOTS_MAN = 0.00001;  // for root maintenance
float KM_ROOTS_RESERVES = 0.1; // for root reserves

float KM_LEAF_SRC = 1.0;     // coefficient for carbon supply from leaf
float KM_INDE_SRC = 1.0;     // coefficient for carbon supply from reserves in internodes
float KM_ROOTS_SRC = 1.0;    // coefficient for carbon supply from reserves in roots

float INDE_SYNTHESIS = 0.1;  // rate of starch synthesis based on structural carbon
float INDE_HYDROLYSIS = 1.0; // rate of starch hydrolysis based on structural carbon (must be <= 1)
float ROOTS_SYNTHESIS = 0.1;  // rate of starch synthesis based on structural carbon
float ROOTS_HYDROLYSIS = 1.0; // rate of starch hydrolysis based on structural carbon (must be <= 1)

float TRANSPORT_RS = 0.025;         // coefficient of resistance along transport pathway (g C / cm^4 s)

float NUM_CANES_DEFOLIATE = 0;

#include "modelinput.h" // must be included after declaration of model parameters!!!

ModelInput parameters;

// Initialization of global variables
Start:
{
  Phase = Develop; // the first phase initializes carbos and flows in organs
  error = 0.0;
  error_count = 0;
  sim_time = 0.0;
  env_time = 0.0;
  vis_time = 0.0;
  out_time = 0.0;
  total_flow_in = 0.0;
  total_flow_out = 0.0;

  parameters.readFromFile("parameters.dat");

  //sran(time(0)); // random seed
  sran(123456);
}

StartEach:
{
  UseGroup(Phase);
  switch (Phase)
  {
    case Develop: // growth and development phase
      if (sim_time > DT)
        NoEnvironment(); // turn environmental step off after budbreak
      Forward();
      if (sim_time < MAX_GROWTH)
      {
        sim_time += DT;
      }
      total_flow_in = 0.0;
      total_flow_out = 0.0;
      break;
    case PropagateLeft: // left pass - accumulation of the resultant E and R
      Backward();
      break;
    case PropagateRight: // right pass - calculation of carbon flows (and error)
      Forward();
      break;  
    case UpdateCarbon: // calculate carbon flow into/from sink/source (and check error level)
      Forward();
      error = 0;
      break;
    case UpdateDemand: // set carbon demand per sink (linearize at the same time)
      Forward();
      break;
    case Debug: // prints out info of all sinks/sources with error > MAX_ERROR
      Printf ("Warning: error > MAXERROR (%g > %g) after %d iterations\n", error, MAXERROR, error_count);
      Printf ("Dumping 'S' modules at sim_time %g:\n", sim_time);
      Forward();
      break;
    case Output: // prints info about one internode
      Forward();
      fruit_carbon_output.reset();
      node_number_output.reset();
      break;
    case LightInterface: // sends leaf information to QuasiMC
      Forward();
      break;
    case Photosynthesis: // computes carbon acquired through photosynthesis
      Forward();
      break;
    case Management: // applies a horticultural management strategy
      Forward();
      break;
  }
}

EndEach:
{   
  switch (Phase) // the control automaton determines next phase type
  {
    case Develop:
      if (sim_time < MAX_GROWTH)
      {
        Phase = Management; // next phase is mangement unless...
        if (sim_time >= env_time)
        {
          Phase = LightInterface;
          #if USE_QUASIMC
            Environment();
          #endif
          env_time += ENV_STEP;
        }
        if (sim_time >= vis_time)
        {
          DisplayFrame();
          vis_time += VIS_STEP;
        }
        error_count = 0;
      }
      else
      {
        Printf ("End of season!\n");
        Stop(); // stop simulation at end of season - isn't this done in StartEach?
      }
      break;
    case LightInterface:
      Phase = Photosynthesis;
      break;
    case Photosynthesis:
      if (sim_time >= out_time)
      {
        Phase = Output;
        out_time += OUT_STEP;
      }
      else
      {
        Phase = Management; // the managemant phase must be "used" every step or MouseIns() won't be inserted
      }
      NoEnvironment();
      break;
    case Output:
      if (fabs(total_flow_in + total_flow_out) >= 0.01/*MAXERROR*/)
      {
        Printf ("WARNING: carbon flow into sinks is not equal to carbon flow out of sources!\n");
        Printf ("flow in: %g\n", total_flow_in);
        Printf ("flow out: %g\n", total_flow_out);
        Printf ("difference: %g\n\n", fabs(total_flow_in + total_flow_out));
        //Phase = Debug;
      }
      //else
      Phase = Management;
      break;
    case Management:
      Phase = UpdateDemand;
      break;
    case UpdateDemand:
      Phase = PropagateLeft;
      break;
    case PropagateLeft:
      Phase = PropagateRight;
      break;
    case PropagateRight:
      Phase = UpdateCarbon;
      break;
    case UpdateCarbon:
      if (error < MAXERROR)
	Phase = Develop;
      else
      {
        if (error_count > 1000)
        {
          Phase = Debug;
        }
        else
        {
          ++error_count;
	  Phase = UpdateDemand;
        }
      }
      break;
    case Debug:
      Stop();
      break;
  }
}

// module type declarations
SegmentData Begin = {{0, INFTY}, 0, {0, 0}, 0, 0, 0, 0, begin}; // start string marker
SegmentData Endd = {{0, INFTY}, 0, {0, 0}, 0, 0, 0, 0, end};   // end string/branch marker
SegmentData InertSegmentData = {{0, INFTY}, 0, {0, 0}, 0, 0, 0, 0, inert};   // inert connector between multiple branches
SegmentData ApexSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, apex};
SegmentData InternodePrimarySegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, internode_primary};
SegmentData InternodeMaintenanceSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, internode_maintenance};
SegmentData InternodeReservesSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, internode_reserves};
SegmentData InternodeHydrolysisSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, internode_hydrolysis};
SegmentData LeafPrimarySegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, leaf_primary};
SegmentData LeafMaintenanceSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, leaf_maintenance};
SegmentData LeafSourceSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, leaf_source};
SegmentData FruitPrimarySegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, fruit_primary};
SegmentData FruitMaintenanceSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, fruit_maintenance};
SegmentData FlowerPrimarySegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, flower_primary};
SegmentData RootsPrimarySegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, roots_primary};
SegmentData RootsMaintenanceSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, roots_maintenance};
SegmentData RootsReservesSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, roots_reserves};
SegmentData RootsHydrolysisSegmentData = {{0, INFTY}, 0.0, {0, 0}, 0, 0, 0, 0.0, roots_hydrolysis};


module S(SegmentData);                                 // a sink/source type
module Roots(float,float,float);                       // roots with attributes: age, volume, volume of reserves
module Internode(float,float,float,float,float,float); // internode with attributes: node,age,order,developmental age,length,radius
module Leaf(float,float,float,float,float,float);      // leaf with attributes: node,age,order,developmental age,area
module Flower(float,float,float);                      // flower with attributes: node,age,diameter
module Fruit(float,float,float);                       // fruit with attributes: node,age,volume
module Apex(float,float,float,float,float);            // apex with attributes: node,age,physiological age,vigour,order
module AxBud(float,float,float,float,float);           // axillary bud that produces flower buds (first 7 nodes) and a regrowth if the shoot is pruned
module CaneBud(int);                                   // bud on cane that produces a shoot and sets it vigour ? shouldn't this just be an axillary bud?
module ProduceMetamer(float,float);                    // decomposed into a new metamer: internode, leaf, and possibly fruit
module Cane(int);                                      // the cane internodes that the shoots are attached to
module Leader(int);                                    // branches with canes
module Vine(int);                                      // creates the main trunk and the roots (first part of the permanent structure)
module P(float,float);                                 // module for sending leaf information to QuasiMC
module J(float);                                       // module for sending current Julian day to QuasiMC

derivation length: 999999;

// only the S modules are considered in context sensitive rules
// the rules for C-TRAM will not work otherwise!
consider: S;

Axiom: J(245) // initial Juilan day - about September 1
       S(Begin)
#if SINGLE_SHOOT
       Right(90) Cane(1)
#else
       Vine(NUM_CANES) // vine with NUM_CANES canes
#endif
       S(Endd)
       ;


/* -------------------------------------------------------------------------- */

group Develop: // production of new metamers, growth of existing organs, and addition of carbon by photosynthesis

Apex(node,age,physio_age,vigour,order) S(sd_apex):
{
  age += DT;
  physio_age += ApexRateofAppearance() * DT;
  sd_apex.update_charge();

  if (node <= MAX_NODES && vigour > 0.0)
  {
    if (physio_age >= 1.0)
    {
      if (ApexGrowthCessation(sd_apex.v, vigour, node))
        produce Apex(node,age,physio_age,0,order); // set vigour to zero; apex aborts

      // else produce a metamer and reset apex's physio_age to zero
      produce ProduceMetamer(node,order) Apex(node+1,age,0.0,vigour,order) S(sd_apex);
    }
  }
  // Apex is either waiting (phyllochron not reached) or has aborted (vigour=0)
  produce Apex(node,age,physio_age,vigour,order) S(sd_apex);
}

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  age += DT;
  dev_age += IndeSpecificGrowthRate() * DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  //sd_maintenance.update_charge();
  sd_maintenance.q = sd_maintenance.i * DT;
  total_flow_in += sd_maintenance.i * DT;

  // now, test if minimum is met? and start senescence for eventual abscission
#if MTNCE_ABSCISSION
  if (fabs(sd_maintenance.q - (IndeMaintenance (sd_primary.q,order) * DT)) > MAXERROR)
  {
    if (sim_time > DT) // ignore for first time step
    {
      Printf ("Abscission INDE(o:%g,n:%g): %g > %g\n", order, node, fabs(sd_maintenance.q - (IndeMaintenance (sd_primary.q,order) * DT)), MAXERROR);
      //produce Internode(node,age,order,dev_age,length,radius) S(InertSegmentData) S(InertSegmentData) S(InertSegmentData) S(InertSegmentData);
    }
  }
#endif

  // accumulate carbon for primary growth
  sd_primary.update_charge();

  length += SinkUnloading(sd_primary.v, 1.0, KM_INDE_PRI) *
            IndePrimaryGrowthRate(dev_age, length) * DT;

  radius += SinkUnloading(sd_primary.v, 1.0, KM_INDE_SEC) *
            IndeSecondaryGrowthRate(dev_age, radius) * DT;

  // for reserves
  sd_reserves.update_charge();
  sd_reserves.q += sd_hydrolysis.i * DT; // some stored carbon is lost to hydrolysis
  if (sd_reserves.q < 0.0)
    sd_reserves.q = 0.0;
  total_flow_out += sd_hydrolysis.i * DT;

  // set resistance along transport pathway (unless it has been girdled)
  if (sd_primary.rs < INFTY)
    sd_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);
  
  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  age += DT;
  dev_age += LeafSpecificGrowthRate() * DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  //sd_maintenance.update_charge();
  sd_maintenance.q = sd_maintenance.i * DT;
  total_flow_in += sd_maintenance.i * DT;

  // now, test if minimum is met? and start senescence for eventual abscission
#if MTNCE_ABSCISSION
  if (fabs(sd_maintenance.q - (LeafMaintenance (sd_primary.q) * DT)) > MAXERROR)
  {
    Printf ("Abscission LEAF(%g): %g > %g\n", node, fabs(sd_maintenance.q - (LeafMaintenance (sd_primary.q) * DT)), MAXERROR);
    //produce Leaf(node,age,order,dev_age,area,carb_acquired) S(InertSegmentData) S(InertSegmentData) S(InertSegmentData);
  }
#endif

  sd_primary.update_charge();

  sd_source.update_charge();

  // photosynthesis
  sd_source.q += LeafCarbonFromPhoto (sd_source.q, sd_primary.q, carb_acquired, area) * DT;

  produce Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source);
}

AxBud(node,age,start_age,vigour,order) S(sd_apex):
{
  age += DT;
  sd_apex.update_charge();

  if (vigour > 0)
  {
    if (order == AX_SHOOT_ORDER && node <= 8)
    {
      // possibly, produce a flower
      if (FlowerDevelopment(node))
      {
        SegmentData sd_flower_primary = FlowerPrimarySegmentData;
        sd_flower_primary.q = INITIAL_FLOWER_CARBON;
        produce Flower(node,0,INITIAL_FLOWER_CARBON*FLOWER_DENSITY) S(sd_flower_primary);
      }
      else // the axillary bud aborts (does not produce flower)
        produce AxBud(node,age,start_age,0,order) S(sd_apex);
    }
    else // a regrowth occurs: Apex with node=1,age=0,physio_age=0,vigour=1,order=axbud's order+1
    {
      // delay the outgrowth of the new shoot
      if (age >= start_age)
        produce Down(INCLINATION_ANGLE) Apex(1,0,0,REGROWTH_VIGOUR,order+1) S(ApexSegmentData);
      else
        produce AxBud(node,age,start_age,vigour,order) S(sd_apex);
    }
  }
  else
  {
    produce AxBud(node,age,start_age,vigour,order) S(sd_apex);
  }

}

Flower(node,age,diameter) S(sd_primary):
{
  age += DT;
  sd_primary.update_charge();

  // Flower could abort if the carbon concentration is too low...
  // because of competition from the leaves (see page 80 in kiwifruit book)
  // Currently, the flower grows for some given amount of time
  if (age >= MAX_FLOWER_AGE)
  {
    SegmentData sd_fruit_primary = FruitPrimarySegmentData;
    // if carbon is not limited, the flower should have INITIAL_FRUIT_CARBON accumulated
    sd_fruit_primary.q = sd_primary.q;

    SegmentData sd_maintenance = FruitMaintenanceSegmentData;
    sd_maintenance.q = FruitMaintenance(sd_primary.q) * DT;

    // at this stage, it should produce fruit
    produce Fruit(node,0,0) S(sd_fruit_primary) S(sd_maintenance);
  }

  produce Flower(node,age,diameter) S(sd_primary);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  age += DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  //sd_maintenance.update_charge();
  sd_maintenance.q = sd_maintenance.i * DT;
  total_flow_in += sd_maintenance.i * DT;

  // now, test if minimum is met? and start senescence for eventual abscission
#if MTNCE_ABSCISSION
  if (fabs(sd_maintenance.q - (FruitMaintenance (sd_primary.q) * DT)) > MAXERROR)
  {
    Printf ("Abscission FRUIT(%g): %g > %g\n", node, fabs(sd_maintenance.q - (FruitMaintenance (sd_primary.q) * DT)), MAXERROR);
    //produce Fruit(node,age,volume) S(InertSegmentData) S(InertSegmentData);
  }
#endif

  // update fruit carbon (size)
  sd_primary.update_charge();

  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}

Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  age += DT;

  // NOTE: carbon is not accumulated for maintenance respiration
  //sd_maintenance.update_charge();
  sd_maintenance.q = sd_maintenance.i * DT;
  total_flow_in += sd_maintenance.i * DT;

  // now, test if minimum is met? and start senescence for eventual abscission
#if MTNCE_ABSCISSION
  if (fabs(sd_maintenance.q - (RootsMaintenance (sd_primary.q) * DT)) > MAXERROR)
  {
    if (sim_time > DT) // ignore for first time step
      Printf ("Abscission ROOTS: %g > %g\n", fabs(sd_maintenance.q - (RootsMaintenance (sd_primary.q) * DT)), MAXERROR);
    //produce;
  }
#endif

  sd_primary.update_charge();

  // update carbon reserves in roots
  sd_reserves.update_charge();
  sd_reserves.q += sd_hydrolysis.i * DT; // some stored carbon is lost to hydrolysis
  if (sd_reserves.q < 0.0)
    sd_reserves.q = 0.0;
  total_flow_out += sd_hydrolysis.i * DT;

  produce Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

S(sd):
{
  switch (sd.type)
  {
    case begin:
    case end:
    case inert:
      break;
    default:
      Printf ("PANIC! Orphan S() module of type %d. Check 'Develop' phase.\n", sd.type);
      break;  
  }
  produce S(sd);
}


decomposition:
maximum depth: 10;

ProduceMetamer(node,order):
{
  // set initial conditions for internode
  SegmentData sd_internode_primary = InternodePrimarySegmentData;
  float radius = INITIAL_INDE_RADIUS;
  float length = IndeInitialLength(node,order);
  sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;
  float inde_age = IndeInitialDevAge();

  SegmentData sd_maintenance = InternodeMaintenanceSegmentData;
  sd_maintenance.q = IndeMaintenance(sd_internode_primary.q,order) * DT;

  SegmentData sd_internode_reserves = InternodeReservesSegmentData;
  sd_internode_reserves.q = 0.001 * sd_internode_primary.q;

  // set the resistance along transport pathway
  // (in the first S module associated with Internode)
  sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

  // set initial conditions for leaf
  SegmentData sd_leaf_primary = LeafPrimarySegmentData;
  sd_leaf_primary.q = LEAF_INITIAL_CARBON;
  float leaf_age = LeafInitialDevAge(node,order);
  float area = Carb2Area(sd_leaf_primary.q);

  // set initial conditions for axillary bud
  float axbud_vigour;
  if (node <= 8 && order == AX_SHOOT_ORDER)
    axbud_vigour = 1.0;
  else 
    axbud_vigour = 0.0;

  // produce an internode and a leaf
  produce RollL(PHYLLOTAXIS)
          Internode(node,0,order,inde_age,length,radius) S(sd_internode_primary)
                                                         S(sd_maintenance)
                                                         S(sd_internode_reserves)
                                                         S(InternodeHydrolysisSegmentData)
          SB() Leaf(node,0,order,leaf_age,area,0) S(sd_leaf_primary)
                                                  S(LeafMaintenanceSegmentData)
                                                  S(LeafSourceSegmentData)
                                                  S(Endd)
          EB()
          // Careful! C-TRAM rules cannot handle more than one branching-point
          S(InertSegmentData)
          SB() AxBud(node,0,REGROWTH_DELAY,axbud_vigour,order) S(ApexSegmentData)
                                                               S(Endd)
          EB()
          ;
}

Cane(num_nodes):
{
  SegmentData sd_internode_primary = InternodePrimarySegmentData;
  SegmentData sd_internode_reserves = InternodeReservesSegmentData;
  SegmentData sd_maintenance = InternodeMaintenanceSegmentData;

  float age = 365;
  float dev_age = 365;
  float radius;
  float length;

#if SINGLE_SHOOT
  length = 15.0;
  radius = 5.0;

  sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;
  sd_maintenance.q = IndeMaintenance(sd_internode_primary.q,CANE_ORDER) * DT;

  sd_internode_reserves.q = CaneIndeInitialReserves (sd_internode_primary.q);
  // check for numerical error (I cannot believe this can even happen!)
  if (sd_internode_primary.q*INDE_RESERVES < sd_internode_reserves.q)
    sd_internode_reserves.q = sd_internode_primary.q * INDE_RESERVES*0.99999;

  // set the resistance along transport pathway
  // (in the first S module associated with Internode)
  sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

  produce Internode(1,age,CANE_ORDER,dev_age,length,radius)
            S(sd_internode_primary)
            S(sd_maintenance)
            S(sd_internode_reserves)
            S(InternodeHydrolysisSegmentData)
            SB() Left(90) GetHead(0,0,0) CaneBud(1) S(Endd) EB()
          ;
#else
  nproduce RollL(ran(1)*PHYLLOTAXIS);
  for (int node = 0; node < num_nodes; node++)
  {
    // set initial conditions for cane internode
    length = CaneIndeInitialLength (node);
    radius = CaneIndeInitialRadius (node, num_nodes);
    sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;

    sd_maintenance.q = IndeMaintenance(sd_internode_primary.q,CANE_ORDER) * DT;

    sd_internode_reserves.q = CaneIndeInitialReserves (sd_internode_primary.q);
    // check for numerical error (I cannot believe this can even happen!)
    if (sd_internode_primary.q*INDE_RESERVES < sd_internode_reserves.q)
      sd_internode_reserves.q = sd_internode_primary.q * INDE_RESERVES*0.99999;

    // set the resistance along transport pathway
    // (in the first S module associated with Internode)
    sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

    nproduce Internode(node+1,age,CANE_ORDER,dev_age,length,radius)
               S(sd_internode_primary)
               S(sd_maintenance)
               S(sd_internode_reserves)
               S(InternodeHydrolysisSegmentData)
             RollL(PHYLLOTAXIS)
             SB() Left(90) GetHead(0,0,0) CaneBud(node+1) S(Endd) EB()
             ;
  }
  produce;
#endif
}

production: // end of decomposition rules

// lpfg allows only one predecessor in decomposition rules and no context sensitive ones,
// so the following production cannot be declared as a decomposition
GetHead(x,y,z) CaneBud(node_number):
{
  // compute angle between potential shoot direction and "up" direction
  V3f shoot_dir;
  V3f up_direction(0,1,0); // up vector for comparison with shoot's direction before bud break
  shoot_dir.Set(x,y,z);
  float angle = up_direction * shoot_dir;

  // check if angle between potential shoot is greater than some minimum needed for budbreak
  if (angle > MIN_BUD_BREAK_ANGLE)
  {
#if SINGLE_SHOOT
    int num_nodes = 7;
#else
    int num_nodes = SetInitialClusterSize();
#endif

    int node;
    for (node = 1; node <= num_nodes; node++)
    {
      nproduce ProduceMetamer(node,AX_SHOOT_ORDER); // with order=AX_SHOOT_ORDER
    }
#if SINGLE_SHOOT // set vigour to 1
    produce Apex(node,0,0,1,AX_SHOOT_ORDER) S(ApexSegmentData);  
#else // set vigour according to node number
    produce Apex(node,0,0,SetShootVigour(node_number),AX_SHOOT_ORDER) S(ApexSegmentData);
#endif
  }
  produce;
}


/* -------------------------------------------------------------------------- */

group Management: // apply horticultural management strategies

/* 
Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  produce Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source);
}

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}

SB() AxBud(node,age,start_age,vigour,order) S(sd0) S(sd1) EB() RollL(ang) MouseIns():
{
  produce SB() AxBud(node,age,age+REGROWTH_DELAY,1,order) S(sd0) S(sd1) EB() RollL(ang) S(Endd) Cut();
}

MouseIns():
{
  produce S(Endd) Cut();
}
*/

/* -------------------------------------------------------------------------- */

group LightInterface: // send leaf geometry to QuasiMC

#if USE_QUASIMC
J(day):
{
  day += ENV_STEP;
  if (day > 365)
    day = day - 365;
  produce E1(0) J(day);
}

Leaf(node,age,order,dev_age,area,carb_acquired):
{
  if (area > 0.0)
    produce Leaf(node,age,order,dev_age,area,carb_acquired) E1(0) P(sqrtf(area/M_PI)*2,sqrtf(area/M_PI)*4);
  else
    produce Leaf(node,age,order,dev_age,area,carb_acquired);
}
#endif

/* -------------------------------------------------------------------------- */

group Photosynthesis: // calculate leaf gross photosynthesis

#if USE_QUASIMC
E1(x) J(day):
{
  produce J(day);
}

Leaf(node,age,order,dev_age,area,carb_acquired) E1(flux_density) P(a,b):
{
  //carb_acquired = Laing1985(flux_density);
  carb_acquired = Greer2004(flux_density);
  produce Leaf(node,age,order,dev_age,area,carb_acquired);
}
#else
Leaf(node,age,order,dev_age,area,carb_acquired):
{
  produce Leaf(node,age,order,dev_age,area,0.35);
}
#endif


/* -------------------------------------------------------------------------- */

group UpdateDemand: // calculate carbon demand for sinks (linearize at the same time)

Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  float demand; // the sink's demand for carbon, linearized! (conductance in electrical circuit)

  // for internode elongation and thickening, demand depends on unloading and relative growth rate
  demand = M_PI * radius * radius * INTERNODE_DENSITY *
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_INDE_PRI) *
           IndePrimaryGrowthRate(dev_age ,length)
           +
           2.0 * M_PI * radius * length * INTERNODE_DENSITY *
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_INDE_SEC) *
           IndeSecondaryGrowthRate(dev_age, radius);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r * (
                    M_PI * radius * radius * INTERNODE_DENSITY *
                    SinkUnloading(sd_primary.v, 1.0, KM_INDE_PRI) *
                    IndePrimaryGrowthRate(dev_age,length)
                    +
                    2.0 * M_PI * radius * length * INTERNODE_DENSITY *
                    SinkUnloading(sd_primary.v, 1.0, KM_INDE_SEC) *
                    IndeSecondaryGrowthRate(dev_age, radius));

  // for internode maintenance, demand depends on unloading and total carbon
  demand = IndeMaintenance(sd_primary.q,order) * 
           dSinkUnloading_dc(sd_maintenance.v, 1.0, KM_INDE_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        IndeMaintenance(sd_primary.q,order) *
                        SinkUnloading(sd_maintenance.v, 1.0, KM_INDE_MAN);

  // for internode reserves, calculate starch synthesis and hydrolysis

  // first, rate of flow of carbos into reserves
  demand = IndeReserveSynthesis (sd_primary.q, sd_reserves.q, INDE_SYNTHESIS) * 
           dSinkUnloading_dc(sd_reserves.v, 1.0, KM_RESERVES);
  sd_reserves.cp.r = (fabs(demand) > EPSILON) ? 1.0/(demand) : INFTY;
  sd_reserves.cp.e = sd_reserves.v - sd_reserves.cp.r *
                     IndeReserveSynthesis (sd_primary.q, sd_reserves.q, INDE_SYNTHESIS) *
                     SinkUnloading(sd_reserves.v, 1.0, KM_RESERVES);

  // notice, hydrolysis demands on amount of carbon in reserves (sd_reserves.q)
  demand = IndeReserveHydrolysis(sd_reserves.q, INDE_HYDROLYSIS) *
           dSourceLoading_dc(sd_hydrolysis.v, KM_INDE_SRC);
  sd_hydrolysis.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_hydrolysis.cp.e = sd_hydrolysis.v - sd_hydrolysis.cp.r *
                       IndeReserveHydrolysis(sd_reserves.q, INDE_HYDROLYSIS) *
                       SourceLoading(sd_hydrolysis.v, KM_INDE_SRC);

  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  float demand; // the sink's carbon demand, linearized! (conductance in electrical circuit)

  // demand for growth depends on unloading function and relative growth rate
  demand = LeafPotentialGrowthRate(sd_primary.q, dev_age, node, order) *
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_LEAF_PRI);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    LeafPotentialGrowthRate(sd_primary.q, dev_age, node, order) *
                    SinkUnloading(sd_primary.v, 1.0, KM_LEAF_PRI);

  // leaf maintenance depends on primary carbon
  demand = LeafMaintenance (sd_primary.q) *
           dSinkUnloading_dc(sd_maintenance.v, 1.0, KM_LEAF_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        LeafMaintenance (sd_primary.q) *
                        SinkUnloading(sd_maintenance.v, 1.0, KM_LEAF_MAN);

  // leaf output depends on carbon acquistion
  demand = LeafCarbonOutput(sd_source.q) * dSourceLoading_dc(sd_source.v, KM_LEAF_SRC);
  sd_source.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_source.cp.e = sd_source.v - sd_source.cp.r *
                                 LeafCarbonOutput(sd_source.q) *
                                 SourceLoading(sd_source.v, KM_LEAF_SRC);

  // convert carbon to leaf area
  area = Carb2Area(sd_primary.q);

  produce Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source);
}

Flower(node,age,diameter) S(sd_primary):
{
  float demand; // The linearized carbon demand 

  demand = FlowerGrowthRate(INITIAL_FRUIT_CARBON) *
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_FLOWER);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    FlowerGrowthRate(INITIAL_FRUIT_CARBON) *
                    SinkUnloading(sd_primary.v, 1.0, KM_FLOWER);

  diameter = sd_primary.q * FLOWER_DENSITY;

  produce Flower(node,age,diameter) S(sd_primary);
}

Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  float demand; // The linearized carbon demand

  demand = FruitGrowthRate(sd_primary.q) * 
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_FRUIT_PRI);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    FruitGrowthRate(sd_primary.q) *
                    SinkUnloading(sd_primary.v, 1.0, KM_FRUIT_PRI);

  demand = FruitMaintenance(sd_primary.q) * 
           dSinkUnloading_dc(sd_maintenance.v, 1.0, KM_FRUIT_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        FruitMaintenance(sd_primary.q) *
                        SinkUnloading(sd_maintenance.v, 1.0, KM_FRUIT_MAN);
  
  volume = sd_primary.q * FRUIT_DENSITY;

  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}

Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  float demand;

  demand = RootsGrowthRate(sd_primary.q) * 
           dSinkUnloading_dc(sd_primary.v, 1.0, KM_ROOTS);

  sd_primary.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_primary.cp.e = sd_primary.v - sd_primary.cp.r *
                    RootsGrowthRate(sd_primary.q) *
                    SinkUnloading(sd_primary.v, 1.0, KM_ROOTS);

  // roots maintenance respiration
  demand = RootsMaintenance(sd_primary.q) * 
           dSinkUnloading_dc(sd_maintenance.v, 1.0, KM_ROOTS_MAN);

  sd_maintenance.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_maintenance.cp.e = sd_maintenance.v - sd_maintenance.cp.r *
                        RootsMaintenance(sd_primary.q) *
                        SinkUnloading(sd_maintenance.v, 1.0, KM_ROOTS_MAN);

  // rate of flow of carbos into reserves
  demand = RootsReserveSynthesis (sd_primary.q, sd_reserves.q, ROOTS_SYNTHESIS) * 
           dSinkUnloading_dc(sd_reserves.v, 1.0, KM_ROOTS_RESERVES);
  sd_reserves.cp.r = (fabs(demand) > EPSILON) ? 1.0/(demand) : INFTY;
  sd_reserves.cp.e = sd_reserves.v - sd_reserves.cp.r *
                     RootsReserveSynthesis (sd_primary.q, sd_reserves.q, ROOTS_SYNTHESIS) *
                     SinkUnloading(sd_reserves.v, 1.0, KM_ROOTS_RESERVES);

  // notice, hydrolysis depends on amount of carbon in reserves (sd_reserves.q)
  demand = RootsReserveHydrolysis(sd_reserves.q, ROOTS_HYDROLYSIS) *
           dSourceLoading_dc(sd_hydrolysis.v, KM_ROOTS_SRC);
  sd_hydrolysis.cp.r = (fabs(demand) > EPSILON) ? 1.0/demand : INFTY;
  sd_hydrolysis.cp.e = sd_hydrolysis.v - sd_hydrolysis.cp.r *
                       RootsReserveHydrolysis(sd_reserves.q, ROOTS_HYDROLYSIS) *
                       SourceLoading(sd_hydrolysis.v, KM_ROOTS_SRC);

  // set Roots attributes
  volume = sd_primary.q * ROOTS_DENSITY; // for visualisation
  res_volume = sd_reserves.q * ROOTS_DENSITY; // also for visualisation

  produce Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}

S(sd):
{
  float demand;
  switch (sd.type)
  {
    case apex:
      sd.cp.r = INFTY;
      sd.cp.e = 0.0;
      break;
    case begin:
    case end:
    case inert:
      break;
    default:
      Printf ("PANIC! Orphan S() module of type %d. Check 'UpdateDemand' phase.\n", sd.type);
      break;  
  }
  produce S(sd);
}


/* -------------------------------------------------------------------------- */

group PropagateLeft: // folding step, reduce neigbouring sinks into one, starting from end

S(sd):
{
  if (sd.type == end)
  {
    sd.ceq = sd.cp;
    produce S(sd);
  }
}

// calculate equivalent of the circuit to the right
S(sd) >> SB() S(sdr2) EB() S(sdr1):
{
  // add serial resistances in both branches
  sdr1.ceq.r += sdr1.rs;
  sdr2.ceq.r += sdr2.rs;

  // Find the circuit equivalent to branches connected in parallel
  LinearData cr = in_parallel(sdr1.ceq, sdr2.ceq);

  // connect in parallel to the current (float) parallel circuit
  sd.ceq = in_parallel(sd.cp, cr);

  produce S(sd);
}

S(sd) >> S(sdr):
{
  // add serial resistance
  sdr.ceq.r += sdr.rs;

  // connect in parallel right equivalent circuit to the current (float) parallel circuit
  sd.ceq = in_parallel(sd.cp, sdr.ceq);

  produce S(sd);
}


/* -------------------------------------------------------------------------- */

group PropagateRight: // unfolding step, find the concentration of carbon along transport pathway

// calculate the voltages; estimate the error
S(sd):
{
  if (sd.type == begin)
  {
    sd.v = sd.ceq.e;
    produce S(sd);
  }
}

S(sdl) << S(sd):
{
  // Find the circuit representing the basal part
  LinearData cl = {sdl.v, sd.rs};

  // connect with the equivalent of the overhanging part
  sd.v = in_parallel(cl,sd.ceq).e;

  produce S(sd);
}

/* -------------------------------------------------------------------------- */

group UpdateCarbon: // update the flow of carbon into each source/sink

S(sd) :
{	
  extern float error;
  float new_i, divisor;	

  if (sd.v >= 0.0)
  {
    // Calculate new current through cp
    divisor = (fabs(sd.cp.r) > EPSILON) ? sd.cp.r : EPSILON;  
    new_i = (sd.v - sd.cp.e)/divisor;

    // Calculate error and save for display purposes
    sd.err = new_i - sd.i;

    // Calculate cumulative error to determine whether it is possible to stop
    error += fabs(sd.err);

    // Update current to be consistent with present values of v, q
    // check for minimum current (so that begin & end components don't have charge)
    if (fabs(new_i) < EPSILON)
      sd.i = 0.0;
    else
      sd.i = new_i;
  }
  else // v < 0! Reset the Newton-Raphson method, with v as zero
  {
    sd.v = 0.0;
    sd.err = 1.0;
    error += 1.0;
  }

  produce S(sd);
}


/* -------------------------------------------------------------------------- */

group Debug: // if error doesn't fall below MAXERROR after many iterations, output debug info

S(sd):
{
  if (sd.type == begin)
    Printf ("BEGIN:\n");
  if (sd.err > MAXERROR)
    Printf ("type: %d = q:%g i:%g v:%g err:%g\n", sd.type, sd.q, sd.i, sd.v, sd.err);

  produce S(sd);
}


/* -------------------------------------------------------------------------- */

group Output: // at every DT time step, write model output to a file


Apex(node,age,physio_age,vigour,order) S(sd_apex):
{
  node_number_output.write(node);
  node_number_output.newList();
  produce Apex(node,age,physio_age,vigour,order) S(sd_apex);
}



Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  if (order == LEADER_ORDER)
  {
    fruit_carbon_output.newList();
    fruit_carbon_output.write(node); // write cane's position (node) along leader
  }
  produce Internode(node,age,order,dev_age,length,radius) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}



Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source):
{
  produce Leaf(node,age,order,dev_age,area,carb_acquired) S(sd_primary) S(sd_maintenance) S(sd_source);
}

AxBud(node,age,start_age,vigour,order) S(sd_apex):
{
  produce AxBud(node,age,start_age,vigour,order) S(sd_apex);
}

Flower(node,age,diameter) S(sd_primary):
{
  produce Flower(node,age,diameter) S(sd_primary);
}



Fruit(node,age,volume) S(sd_primary) S(sd_maintenance):
{
  fruit_carbon_output.write (sd_primary.q);
  produce Fruit(node,age,volume) S(sd_primary) S(sd_maintenance);
}



Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis):
{
  produce Roots(age,volume,res_volume) S(sd_primary) S(sd_maintenance) S(sd_reserves) S(sd_hydrolysis);
}


/* -------------------------------------------------------------------------- */


group 0:

decomposition:

// the permanent structure decomposition rules have to be declared here in group 0
// because the budbreak rule uses a GetHead() module
// that requires another pass through the L-system string (otherwise the turtle heading
// is returned as all zeros).

Vine(num_canes):
{
  SegmentData sd_roots_primary = RootsPrimarySegmentData;
  SegmentData sd_roots_reserves = RootsReservesSegmentData;
  SegmentData sd_roots_maintenance = RootsMaintenanceSegmentData;
  SegmentData sd_internode_primary = InternodePrimarySegmentData;
  SegmentData sd_internode_reserves = InternodeReservesSegmentData;
  SegmentData sd_internode_maintenance = InternodeMaintenanceSegmentData;

  float age = 365.0 * 3.0;
  float dev_age = 365.0 * 3.0;
  float radius;
  float length;

  // produce the roots
  sd_roots_primary.q = INIT_ROOTS_STORAGE;
  sd_roots_maintenance.q = RootsMaintenance(sd_roots_primary.q) * DT;
  sd_roots_reserves.q = RootsInitialReserves(sd_roots_primary.q);
  // check for numerical error
  if (sd_roots_primary.q*ROOTS_RESERVES < sd_roots_reserves.q)
    sd_roots_reserves.q = sd_roots_primary.q * ROOTS_RESERVES*0.99999;

  nproduce Roots(age,sd_roots_primary.q*ROOTS_DENSITY,sd_roots_reserves.q*ROOTS_DENSITY)
             S(sd_roots_primary)
             S(sd_roots_maintenance)
             S(sd_roots_reserves)
             S(RootsHydrolysisSegmentData);

  // produce the main trunk
  for (int node = 0; node < TRUNK_NUM_NODES; node++)
  {
    // set initial conditions
    length = TRUNK_INDE_LEN;//CaneIndeInitialLength (node);
    radius = CaneIndeInitialRadius (node, TRUNK_NUM_NODES) * 5.0;

    sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;

    sd_internode_maintenance.q = IndeMaintenance(sd_internode_primary.q,TRUNK_ORDER) * DT;

    sd_internode_reserves.q = CaneIndeInitialReserves (sd_internode_primary.q);
    // check for numerical error (I cannot believe this can even happen!)
    if (sd_internode_primary.q*INDE_RESERVES < sd_internode_reserves.q)
      sd_internode_reserves.q = sd_internode_primary.q * INDE_RESERVES*0.99999;

    // set the resistance along transport pathway
    // (in the first S module associated with Internode)
    sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

    nproduce Internode(node+1,age,TRUNK_ORDER,dev_age,length,radius)
               S(sd_internode_primary)
               S(sd_internode_maintenance)
               S(sd_internode_reserves)
               S(InternodeHydrolysisSegmentData);
  }

  // produce the leaders
  produce SB() Right(90) Leader(num_canes/2) S(Endd) EB()
          S(InertSegmentData) // Careful! C-TRAM rules cannot handle more than one branching-point
          SB() Left(90) RollL(180) Leader(num_canes/2) S(Endd) EB();
}

Leader(num_canes):
{
  SegmentData sd_internode_primary = InternodePrimarySegmentData;
  SegmentData sd_internode_maintenance = InternodeMaintenanceSegmentData;
  SegmentData sd_internode_reserves = InternodeReservesSegmentData;

  float age = 365.0 * 2.0;
  float dev_age = 365.0 * 2.0;
  float radius;
  float length;

  //nproduce RollL(ran(1)*PHYLLOTAXIS);
  for (int node = 0; node < num_canes; node++)
  {
    // set initial conditions
    length = 15.0;//4.*CaneIndeInitialLength (node);
    radius = CaneIndeInitialRadius (node, num_canes) * 2.0;
    sd_internode_primary.q = INTERNODE_DENSITY * M_PI * radius * radius * length;

    sd_internode_maintenance.q = IndeMaintenance(sd_internode_primary.q,LEADER_ORDER) * DT;

    sd_internode_reserves.q = CaneIndeInitialReserves (sd_internode_primary.q);
    // check for numerical error (I cannot believe this can even happen!)
    if (sd_internode_primary.q*INDE_RESERVES < sd_internode_reserves.q)
      sd_internode_reserves.q = sd_internode_primary.q * INDE_RESERVES*0.99999;

    // set the resistance along transport pathway
    // (in the first S module associated with Internode)
    sd_internode_primary.rs = IndeFlowResistance(length, radius, TRANSPORT_RS);

    nproduce Internode(node+1,age,LEADER_ORDER,dev_age,length,radius)
               S(sd_internode_primary)
               S(sd_internode_maintenance)
               S(sd_internode_reserves)
               S(InternodeHydrolysisSegmentData)
             //RollL(PHYLLOTAXIS)
             RollL(180)
             SB() Up(90) GetHead(0,0,0) Cane(CANE_NUM_NODES) S(Endd) EB()
             ;
  }
  produce;
}


interpretation:

Apex(node,age,physio_age,vigour,order):
{
  produce SetColor(15) Sphere(INITIAL_INDE_RADIUS);
}

Internode(node,age,order,dev_age,length,radius):
{
  static float last_radius;
  int colour = age >= 365 ? 11 : 15;  // mature internodes are brown; young are green

  // checks to save radius of last internode so shoot grows from outside parent branch
  if (order == LEADER_ORDER)
    last_radius = radius;
  else if (order == CANE_ORDER)
  {
    if (node == 1)
      nproduce f(last_radius); // move first internode of cane out of leader
    last_radius = radius;

    // set elasticity of cane - so they bend due to gravity
#if TBAR
    nproduce SetElasticity(0,0.0025*(node-1));   
#endif
  }
  else if (order == AX_SHOOT_ORDER)
  {
    if (node == 1)
      nproduce f(last_radius); // move first internode of shoot out of cane
    last_radius = radius;

    // set elasticity of shoots - so they bend due to gravity
    nproduce SetElasticity(0,0.001*node);    
  }
  else // a regrowth from a pruned shoot
  {
    // set elasticity of shoots - so they bend due to gravity
    nproduce SetElasticity(0,0.001*node);    
  }

  produce SetColor(colour) SetWidth(radius*2.0) F(length);
}

Leaf(node,age,order,dev_age,area,carb_acquired):
{
  float radius = sqrtf(area/M_PI); // for visualisation as a disk (area=pi*radius^2)
  produce SetColor(14)
        
          SetElasticity(0,0.0) SetElasticity(1,0.2)
          Down(INCLINATION_ANGLE)

          // leaf stalk
          SetWidth(radius*0.05)
          F(radius*0.25) F(radius*0.25) F(radius*0.25) F(radius*0.25)
          
          RollToVert() // set the turtle's up vector to point up

          // leaf
          Down(90)
          // Circle(radius) // the following surface should be the same size as the circle
          SB() Surface(0,radius*0.8) EB()
          SetElasticity(1,0.0)
          ;
}

Flower(node,age,diameter):
{
  // peduncle length is 10 to 40 mm
  // flower diameter is 35 to 68 mm

  float radius = diameter*0.5f;

  produce SB() //Peduncle
               Down(110) SetColor(15) SetWidth(diameter*0.05) F(diameter*0.6)
               //Petals
               Down(90) SetColor(12) Circle(radius)
               //stamen (note: no stigma)
               Up(90) f(diameter*0.01) Down(90) SetColor(5) Circle(radius*0.25)
          EB();
}

Fruit(node,age,volume):
{
  float radius, height;
  float final_volume = MAX_FRUIT_CARBON * FRUIT_DENSITY;

  // max length of the fruit is 6.5cm
  // max circumference is 18cm => max radius is 2.86cm

  radius = powf(0.44*volume*M_1_PI,0.3333);
  height = 2.2727 * radius;

  produce SB() SetElasticity(0,0.25*volume/final_volume)
               Down(110) SetColor(15) SetWidth(0.25f)
               // leave peduncle from flower (6.8 cm)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               F(6.8*0.5*0.25)
               SetColor(13) f(radius) Sphere(radius)
                            SetWidth(radius*2.0) F(height-2.0*radius)
                            Sphere(radius)
          EB();
}

Roots(age,volume,res_volume):
{
  // for visualisation, assume the radius of the root bucket is 10 cm
  const float radius = 20.0;
  float length = volume / (M_PI*radius*radius);
  float res_length = res_volume / (M_PI*radius*radius);

  produce SB() SetColor(10) 
               Down(90) Circle(radius) Up(90)
               SetWidth(radius*2.0) F(-length)

               SetColor(16) 
               Down(90) Circle(radius) Up(90)
               SetWidth(radius*2.0) F(-res_length)
          EB();
}



